function [f1, f2, f3, f4, f5, f6, f7, f8] = EightDirGrad(f)
    % Eight Directional Gradeints
    % input argument: the gray-level image 'f'
    % output arguments: the decomposed gradient matrices 'f1', 'f2', 'f3', 'f4', 'f5', 'f6', 'f7', and 'f8',

    f = im2double(f);
    [h, w] = size(f);

    Myi = [1 0 -1;
           2 0 -2;
           1 0 -1]; % inverse mask
    Myi = 0.125.*Myi;
    Mxi = Myi';

    [roundedf_roundedx, roundedf_roundedy] = Filtering(f, Myi, Mxi, h, w); % Filtering

    f1 = zeros(h, w);
    f2 = zeros(h, w);
    f3 = zeros(h, w);
    f4 = zeros(h, w);
    f5 = zeros(h, w);
    f6 = zeros(h, w);
    f7 = zeros(h, w);
    f8 = zeros(h, w);
    for r = 1:1:h
        for c = 1:1:w
            g_v = [roundedf_roundedx(r, c); roundedf_roundedy(r, c)];
            g_v_abs = abs(g_v);
            g_v_abs_n2 = sqrt(g_v_abs(1)*g_v_abs(1) + g_v_abs(2)*g_v_abs(2));
            if g_v_abs_n2 ~= 0
                a1 = abs(g_v_abs(1) - g_v_abs(2))/g_v_abs_n2;
                a2 = (sqrt(2)*min(g_v_abs(1), g_v_abs(2)))/g_v_abs_n2;
            else
                a1 = 0;
                a2 = 0;
            end
            if (g_v(1) >= 0 && g_v_abs(2) <= g_v_abs(1))
                f1(r, c) = a1; % d1 = 'D1'
            elseif (g_v(1) < 0 && g_v_abs(2) <= g_v_abs(1))
                f5(r, c) = a1; % d1 = 'D5'
            elseif (g_v(1) >= 0 && g_v_abs(2) > g_v_abs(1))
                f7(r, c) = a1; % d1 = 'D7'
            elseif (g_v(1) < 0 && g_v_abs(2) > g_v_abs(1))
                f3(r, c) = a1; % d1 = 'D3'
            end
            if (g_v(1) >= 0 && g_v(2) >= 0)
                f8(r, c) = a2; % d2 = 'D8'
            elseif (g_v(1) >= 0 && g_v(2) < 0)
                f2(r, c) = a2; % d2 = 'D2'
            elseif (g_v(1) < 0 && g_v(2) < 0)
                f4(r, c) = a2; % d2 = 'D4'
            elseif (g_v(1) < 0 && g_v(2) >= 0)
                f6(r, c) = a2; % d2 = 'D6'
            end
        end
    end
    
end

