function [Img, M, Irec] = MASCAR(I)
    % Measurement-Adaptive Sampling Cellular Automaton Recovery (MASCAR)
    % input argument: the input image 'I'
    % output arguments:
    % the original image 'Img'
    % the sampling mask 'Smask'
    % the recovered image 'Irec'
       
    [I, rm] = zeropadding(I); % zero padding
    
    if size(I, 3) == 3
        Img_gray = rgb2gray(I);
        Img = im2double(Img_gray);
    else
        Img_gray = I;
        Img = im2double(I);
    end
    
    M = SamplingMaskGenerator(Img_gray); % Sampling Mask Generator

    SImg = M.*Img; % Subsampled image
    
    Irec = CellularAutomatonRecovery(SImg, M); % Cellular Automaton-based Recovery

    [Img, M, Irec] = cropping(Img, M, Irec, rm); % cropping
    
end

