function QMsl = MakingQT(QF)
    % Making Quantization Table
    % input argument: Quality Factor 'QF'
    % output argument: Quantization Table 'QMsl'
    
    if QF == 0
        QF = 0.00001; % for prevention from being Inf
    end
    QMbl = [16 11 10 16  24  40  51  61;
            12 12 14 19  26  58  60  55;
            14 13 16 24  40  57  69  56;
            14 17 22 29  51  87  80  62;
            18 22 37 56  68 109 103  77;
            24 35 55 64  81 104 113  92;
            49 64 78 87 103 121 120 101;
            72 92 95 98 112 100 103  99]; % Base Quantization Matrix for Luminance
    if QF < 50
        SF = 5000 / QF;
    else
        SF = 200 - 2*QF;
    end
    QMsl = zeros(8, 8); % Scaled Quantization Matrix for Luminance
    for y = 1:1:8
        for x = 1:1:8
            QMsl(y, x) = floor((SF*QMbl(y, x) + 50) / 100);
            if QMsl(y, x) == 0
                QMsl(y, x) = 1;
            end     
        end
    end
    
end

