function M = SamplingMaskGenerator(f)
    % Sampling Mask Generator
    % input argument: the gray-level image 'f'
    % output argument: the binary sampling mask 'M'

    Lb = 8; % Length of block
    Hmax = 6; % maximum entropy
        
    c = 1.3;
    DC_term = 2.8;
    
    tau = 0.9;
    
    [Hi, Wi] = size(f); % Height and Width of the image

    M = zeros(Hi, Wi);   
    
    [F1, F2, F3, F4, F5, F6, F7, F8] = EightDirGrad(f);
          
    Pattvl = [0 0 0 0 0 0 0 0;
              0 0 0 0 0 0 0 0;
              0 0 0 0 0 0 0 0;
              0 0 0 1 0 0 0 0;
              0 0 0 0 0 0 0 0;
              0 0 0 0 0 0 0 0;
              0 0 0 0 0 0 0 0;
              0 0 0 0 0 0 0 0]; % Nnzc = 1
     
     Pattl = [1 0 0 0 0 0 0 0;
              0 0 0 0 0 0 0 0;
              0 0 0 0 0 0 0 0;
              0 0 0 0 0 0 0 0;
              0 0 0 0 1 0 0 0;
              0 0 0 0 0 0 0 0;
              0 0 0 0 0 0 0 0;
              0 0 0 0 0 0 0 0]; % Nnzc = 2
      
     Pattb = [1 0 0 0 0 0 0 0;
              0 0 0 0 0 0 0 0;
              0 0 0 0 0 0 1 0;
              0 0 0 0 0 0 0 0;
              1 0 0 0 0 0 0 0;
              0 0 0 0 0 0 0 0;
              0 0 0 0 0 0 1 0;
              0 0 0 0 0 0 0 0]; % Nnzc = 4
      
     Patth = [1 0 0 0 1 0 0 0;
              0 0 0 0 0 0 0 0;
              0 0 1 0 0 0 1 0;
              0 0 0 0 0 0 0 0;
              1 0 0 0 1 0 0 0;
              0 0 0 0 0 0 0 0;
              0 0 1 0 0 0 1 0;
              0 0 0 0 0 0 0 0]; % Nnzc = 8

    Patthh = [1 0 1 0 1 0 1 0;
              0 0 0 0 0 0 0 0;
              0 1 0 1 0 1 0 1;
              0 0 0 0 0 0 0 0;
              1 0 1 0 1 0 1 0;
              0 0 0 0 0 0 0 0;
              0 1 0 1 0 1 0 1;
              0 0 0 0 0 0 0 0]; % Nnzc = 16

    for y = 1:Lb:Hi
        for x = 1:Lb:Wi
            B = f(y:y+Lb-1, x:x+Lb-1);

            glcm = graycomatrix(B, 'NumLevels', 8, 'Offset', [0, 1]);
            glcm_pmf = glcm./sum(glcm(:));
            p = glcm_pmf(:);
            E = 0;
            for i = 1:1:length(p)
                if p(i) ~= 0
                    E = p(i)*log2(1/p(i)) + E;
                end
            end
            Atex = E/Hmax; % Amplitude of texture

            TF = 100*Atex; % Texture Factor, which is a number between 0 and 100.

            QM = MakingQT(TF); % Making Quantization Table

            B_DCT = dct2(B);

            B_DCT_thr = round(B_DCT ./ QM);

            Snumber = length(find(B_DCT_thr(:) ~= 0)); % sparsity number

            if Snumber ~= 0
                SR = round(c*Snumber.*log10(DC_term*((Lb*Lb)./Snumber)));
            else % elseif Snumber == 0
                SR = 0;
            end

            if (TF >= 0 && TF < 10)
                UM = Pattvl;
            elseif (TF >= 10 && TF < 25)
                UM = Pattl;
            elseif (TF >= 25 && TF < 45)    
                UM = Pattb;
            elseif (TF >= 45 && TF < 70)
                UM = Patth;
            else  % elseif (TF >= 70 && TF <= 100)
                UM = Patthh;
            end
            [LocBlank_r, LocBlank_c, LocBlank_v]  = find(UM == 0); 
            L = [LocBlank_r, LocBlank_c];
            Nz_patt = length(LocBlank_v);
            
            if SR > Nz_patt % truncation
                SR = Nz_patt;
            end
            
            L = DataShuffling(L); % Data Shuffling

            LoNZE = transpose(randperm(Nz_patt, SR)); 
            RM = zeros(Lb, Lb);
            for k = 1:SR
                temp = L(LoNZE(k), :);
                rind = temp(1);
                cind = temp(2);
                RM(rind, cind) = 1;
            end
 
            NUM = zeros(Lb, Lb); % Non-Uniform Mask
            SparLoc = sparsitylocations(B_DCT_thr); % localizing sparsity locations 
            if (strcmp(SparLoc, '11000') | strcmp(SparLoc, '10100') | strcmp(SparLoc, '10010') | strcmp(SparLoc, '11010') | strcmp(SparLoc, '10110')) == 1 
                f1 = F1(y:y+Lb-1, x:x+Lb-1);
                f2 = F2(y:y+Lb-1, x:x+Lb-1);
                f3 = F3(y:y+Lb-1, x:x+Lb-1);
                f4 = F4(y:y+Lb-1, x:x+Lb-1);
                f5 = F5(y:y+Lb-1, x:x+Lb-1);
                f6 = F6(y:y+Lb-1, x:x+Lb-1);
                f7 = F7(y:y+Lb-1, x:x+Lb-1);
                f8 = F8(y:y+Lb-1, x:x+Lb-1);
                
                F_patch(:, :, 1) = f1;
                F_patch(:, :, 2) = f2;
                F_patch(:, :, 3) = f3;
                F_patch(:, :, 4) = f4;
                F_patch(:, :, 5) = f5;
                F_patch(:, :, 6) = f6;
                F_patch(:, :, 7) = f7;
                F_patch(:, :, 8) = f8;
                
                [~, ind_dom_dir] = sort([sum(f1(:)), sum(f2(:)), sum(f3(:)), sum(f4(:)), sum(f5(:)), sum(f6(:)), sum(f7(:)), sum(f8(:))], 'descend');
                ABS = sqrt(F_patch(:, :, ind_dom_dir(1)).*F_patch(:, :, ind_dom_dir(1)) + F_patch(:, :, ind_dom_dir(2)).*F_patch(:, :, ind_dom_dir(2)));             
                ABS = imnorm(ABS); % image normalization between 0 and 1
                 
                NUM = ABS > tau;
            end % end if      
            M(y:y+Lb-1, x:x+Lb-1) = UM | RM | NUM;   
     
        end
    end
   
end

